@@ -4,7 +4,7 @@
  *  tests on these numbers like is_integer() etc... */
 
 /*
- *  GiNaC Copyright (C) 1999-2011 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -68,6 +68,11 @@ static unsigned exam_numeric1()
 		     << " erroneously not recognized as complex rational" << endl;
 		++result;
 	}
+	if (test_crat.info(info_flags::nonnegative)) {
+		clog << test_crat
+		     << " erroneously recognized as non-negative number" << endl;
+		++result;
+	}
 	
 	int i = numeric(1984).to_int();
 	if (i-1984) {
