@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's initially known functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "ex.h"
 #include "constant.h"
 #include "lst.h"
+#include "fderivative.h"
 #include "matrix.h"
 #include "mul.h"
 #include "power.h"
@@ -66,6 +67,19 @@ static ex conjugate_conjugate(const ex & arg)
 	return arg;
 }
 
+// If x is real then U.diff(x)-I*V.diff(x) represents both conjugate(U+I*V).diff(x) 
+// and conjugate((U+I*V).diff(x))
+static ex conjugate_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return conjugate(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(conjugate(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 static ex conjugate_real_part(const ex & arg)
 {
 	return arg.real_part();
@@ -76,8 +90,47 @@ static ex conjugate_imag_part(const ex & arg)
 	return -arg.imag_part();
 }
 
+static bool func_arg_info(const ex & arg, unsigned inf)
+{
+	// for some functions we can return the info() of its argument
+	// (think of conjugate())
+	switch (inf) {
+		case info_flags::polynomial:
+		case info_flags::integer_polynomial:
+		case info_flags::cinteger_polynomial:
+		case info_flags::rational_polynomial:
+		case info_flags::real:
+		case info_flags::rational:
+		case info_flags::integer:
+		case info_flags::crational:
+		case info_flags::cinteger:
+		case info_flags::even:
+		case info_flags::odd:
+		case info_flags::prime:
+		case info_flags::crational_polynomial:
+		case info_flags::rational_function:
+		case info_flags::algebraic:
+		case info_flags::positive:
+		case info_flags::negative:
+		case info_flags::nonnegative:
+		case info_flags::posint:
+		case info_flags::negint:
+		case info_flags::nonnegint:
+		case info_flags::has_indices:
+			return arg.info(inf);
+	}
+	return false;
+}
+
+static bool conjugate_info(const ex & arg, unsigned inf)
+{
+	return func_arg_info(arg, inf);
+}
+
 REGISTER_FUNCTION(conjugate_function, eval_func(conjugate_eval).
                                       evalf_func(conjugate_evalf).
+                                      expl_derivative_func(conjugate_expl_derivative).
+                                      info_func(conjugate_info).
                                       print_func<print_latex>(conjugate_print_latex).
                                       conjugate_func(conjugate_conjugate).
                                       real_part_func(conjugate_real_part).
@@ -121,8 +174,21 @@ static ex real_part_imag_part(const ex & arg)
 	return 0;
 }
 
+// If x is real then Re(e).diff(x) is equal to Re(e.diff(x)) 
+static ex real_part_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return real_part_function(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(real_part(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 REGISTER_FUNCTION(real_part_function, eval_func(real_part_eval).
                                       evalf_func(real_part_evalf).
+                                      expl_derivative_func(real_part_expl_derivative).
                                       print_func<print_latex>(real_part_print_latex).
                                       conjugate_func(real_part_conjugate).
                                       real_part_func(real_part_real_part).
@@ -166,8 +232,21 @@ static ex imag_part_imag_part(const ex & arg)
 	return 0;
 }
 
+// If x is real then Im(e).diff(x) is equal to Im(e.diff(x)) 
+static ex imag_part_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return imag_part_function(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(imag_part(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 REGISTER_FUNCTION(imag_part_function, eval_func(imag_part_eval).
                                       evalf_func(imag_part_evalf).
+                                      expl_derivative_func(imag_part_expl_derivative).
                                       print_func<print_latex>(imag_part_print_latex).
                                       conjugate_func(imag_part_conjugate).
                                       real_part_func(imag_part_real_part).
@@ -194,12 +273,58 @@ static ex abs_eval(const ex & arg)
 	if (arg.info(info_flags::nonnegative))
 		return arg;
 
+	if (arg.info(info_flags::negative) || (-arg).info(info_flags::nonnegative))
+		return -arg;
+
 	if (is_ex_the_function(arg, abs))
 		return arg;
 
+	if (is_ex_the_function(arg, exp))
+		return exp(arg.op(0).real_part());
+
+	if (is_exactly_a<power>(arg)) {
+		const ex& base = arg.op(0);
+		const ex& exponent = arg.op(1);
+		if (base.info(info_flags::positive) || exponent.info(info_flags::real))
+			return pow(abs(base), exponent.real_part());
+	}
+
+	if (is_ex_the_function(arg, conjugate_function))
+		return abs(arg.op(0));
+
+	if (is_ex_the_function(arg, step))
+		return arg;
+
 	return abs(arg).hold();
 }
 
+static ex abs_expand(const ex & arg, unsigned options)
+{
+	if ((options & expand_options::expand_transcendental)
+		&& is_exactly_a<mul>(arg)) {
+		exvector prodseq;
+		prodseq.reserve(arg.nops());
+		for (const_iterator i = arg.begin(); i != arg.end(); ++i) {
+			if (options & expand_options::expand_function_args)
+				prodseq.push_back(abs(i->expand(options)));
+			else
+				prodseq.push_back(abs(*i));
+		}
+		return (new mul(prodseq))->setflag(status_flags::dynallocated | status_flags::expanded);
+	}
+
+	if (options & expand_options::expand_function_args)
+		return abs(arg.expand(options)).hold();
+	else
+		return abs(arg).hold();
+}
+
+static ex abs_expl_derivative(const ex & arg, const symbol & s)
+{
+	ex diff_arg = arg.diff(s);
+	return (diff_arg*arg.conjugate()+arg*diff_arg.conjugate())/2/abs(arg);
+}
+
 static void abs_print_latex(const ex & arg, const print_context & c)
 {
 	c.s << "{|"; arg.print(c); c.s << "|}";
@@ -212,7 +337,7 @@ static void abs_print_csrc_float(const ex & arg, const print_context & c)
 
 static ex abs_conjugate(const ex & arg)
 {
-	return abs(arg);
+	return abs(arg).hold();
 }
 
 static ex abs_real_part(const ex & arg)
@@ -227,14 +352,47 @@ static ex abs_imag_part(const ex& arg)
 
 static ex abs_power(const ex & arg, const ex & exp)
 {
-	if (arg.is_equal(arg.conjugate()) && is_a<numeric>(exp) && ex_to<numeric>(exp).is_even())
-		return power(arg, exp);
-	else
+	if ((is_a<numeric>(exp) && ex_to<numeric>(exp).is_even()) || exp.info(info_flags::even)) {
+		if (arg.info(info_flags::real) || arg.is_equal(arg.conjugate()))
+			return power(arg, exp);
+		else
+			return power(arg, exp/2)*power(arg.conjugate(), exp/2);
+	} else
 		return power(abs(arg), exp).hold();
 }
 
+bool abs_info(const ex & arg, unsigned inf)
+{
+	switch (inf) {
+		case info_flags::integer:
+		case info_flags::even:
+		case info_flags::odd:
+		case info_flags::prime:
+			return arg.info(inf);
+		case info_flags::nonnegint:
+			return arg.info(info_flags::integer);
+		case info_flags::nonnegative:
+		case info_flags::real:
+			return true;
+		case info_flags::negative:
+			return false;
+		case info_flags::positive:
+			return arg.info(info_flags::positive) || arg.info(info_flags::negative);
+		case info_flags::has_indices: {
+			if (arg.info(info_flags::has_indices))
+				return true;
+			else
+				return false;
+		}
+	}
+	return false;
+}
+
 REGISTER_FUNCTION(abs, eval_func(abs_eval).
                        evalf_func(abs_evalf).
+                       expand_func(abs_expand).
+                       expl_derivative_func(abs_expl_derivative).
+                       info_func(abs_info).
                        print_func<print_latex>(abs_print_latex).
                        print_func<print_csrc_float>(abs_print_csrc_float).
                        print_func<print_csrc_double>(abs_print_csrc_float).
@@ -398,7 +556,7 @@ static ex csgn_power(const ex & arg, const ex & exp)
 {
 	if (is_a<numeric>(exp) && exp.info(info_flags::positive) && ex_to<numeric>(exp).is_integer()) {
 		if (ex_to<numeric>(exp).is_odd())
-			return csgn(arg);
+			return csgn(arg).hold();
 		else
 			return power(csgn(arg), _ex2).hold();
 	} else
@@ -489,7 +647,7 @@ static ex eta_series(const ex & x, const ex & y,
 
 static ex eta_conjugate(const ex & x, const ex & y)
 {
-	return -eta(x, y);
+	return -eta(x, y).hold();
 }
 
 static ex eta_real_part(const ex & x, const ex & y)
@@ -648,7 +806,7 @@ static ex Li2_conjugate(const ex & x)
 	// conjugate(Li2(x))==Li2(conjugate(x)) unless on the branch cuts which
 	// run along the positive real axis beginning at 1.
 	if (x.info(info_flags::negative)) {
-		return Li2(x);
+		return Li2(x).hold();
 	}
 	if (is_exactly_a<numeric>(x) &&
 	    (!x.imag_part().is_zero() || x < *_num1_p)) {
@@ -687,7 +845,7 @@ static ex zetaderiv_eval(const ex & n, const ex & x)
 	if (n.info(info_flags::numeric)) {
 		// zetaderiv(0,x) -> zeta(x)
 		if (n.is_zero())
-			return zeta(x);
+			return zeta(x).hold();
 	}
 	
 	return zetaderiv(n, x).hold();
@@ -870,11 +1028,15 @@ static ex Order_imag_part(const ex & x)
 	return Order(x).hold();
 }
 
-// Differentiation is handled in function::derivative because of its special requirements
+static ex Order_expl_derivative(const ex & arg, const symbol & s)
+{
+	return Order(arg.diff(s));
+}
 
 REGISTER_FUNCTION(Order, eval_func(Order_eval).
                          series_func(Order_series).
                          latex_name("\\mathcal{O}").
+                         expl_derivative_func(Order_expl_derivative).
                          conjugate_func(Order_conjugate).
                          real_part_func(Order_real_part).
                          imag_part_func(Order_imag_part));
@@ -1007,9 +1169,24 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 	do {
 		xxprev = xx[side];
 		fxprev = fx[side];
-		xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
-		fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
-		if ((side==0 && xx[0]<xxprev) || (side==1 && xx[1]>xxprev) || xx[0]>xx[1]) {
+		ex dx_ = ff.subs(x == xx[side]).evalf();
+		if (!is_a<numeric>(dx_))
+			throw std::runtime_error("fsolve(): function derivative does not evaluate numerically");
+		xx[side] += ex_to<numeric>(dx_);
+		// Now check if Newton-Raphson method shot out of the interval 
+		bool bad_shot = (side == 0 && xx[0] < xxprev) || 
+				(side == 1 && xx[1] > xxprev) || xx[0] > xx[1];
+		if (!bad_shot) {
+			// Compute f(x) only if new x is inside the interval.
+			// The function might be difficult to compute numerically
+			// or even ill defined outside the interval. Also it's
+			// a small optimization. 
+			ex f_x = f.subs(x == xx[side]).evalf();
+			if (!is_a<numeric>(f_x))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically");
+			fx[side] = ex_to<numeric>(f_x);
+		}
+		if (bad_shot) {
 			// Oops, Newton-Raphson method shot out of the interval.
 			// Restore, and try again with the other side instead!
 			xx[side] = xxprev;
@@ -1017,8 +1194,16 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 			side = !side;
 			xxprev = xx[side];
 			fxprev = fx[side];
-			xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf());
-			fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf());
+
+			ex dx_ = ff.subs(x == xx[side]).evalf();
+			if (!is_a<numeric>(dx_))
+				throw std::runtime_error("fsolve(): function derivative does not evaluate numerically [2]");
+			xx[side] += ex_to<numeric>(dx_);
+
+			ex f_x = f.subs(x==xx[side]).evalf();
+			if (!is_a<numeric>(f_x))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically [2]");
+			fx[side] = ex_to<numeric>(f_x);
 		}
 		if ((fx[side]<0 && fx[!side]<0) || (fx[side]>0 && fx[!side]>0)) {
 			// Oops, the root isn't bracketed any more.
@@ -1042,7 +1227,10 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 			static const double secant_weight = 0.984375;  // == 63/64 < 1
 			numeric xxmid = (1-secant_weight)*0.5*(xx[0]+xx[1])
 			    + secant_weight*(xx[0]+fx[0]*(xx[0]-xx[1])/(fx[1]-fx[0]));
-			numeric fxmid = ex_to<numeric>(f.subs(x==xxmid).evalf());
+			ex fxmid_ = f.subs(x == xxmid).evalf();
+			if (!is_a<numeric>(fxmid_))
+				throw std::runtime_error("fsolve(): function does not evaluate numerically [3]");
+			numeric fxmid = ex_to<numeric>(fxmid_);
 			if (fxmid.is_zero()) {
 				// Luck strikes...
 				return xxmid;
